/* KallistiGL for KOS 1.1.3

   gldraw.c
   (c)2001 Dan Potter
*/

static char id[] = "KGL $Id: gldraw.c,v 1.1 2001/09/09 23:58:36 bard Exp $";

#include <GL/gl.h>
#include <GL/glu.h>
#include "glinternal.h"

#include <stdio.h>
#include <dc/ta.h>
#include <dcutils/3dutils.h>

/* Drawing functions */

/* Draw buffer; this is going to get much more complicated, but for now we
   just want to have a buffer to store intermediate points. */
static vertex_oc_t dbuf_col[8];
static vertex_ot_t dbuf_txr[8];
static int dbuf_top;

/* Current vertex attributes (move to global?) */
static GLclampd vert_r = 1.0f, vert_g = 1.0f, vert_b = 1.0f, vert_a = 1.0f;
static GLfloat vert_u = 0.0f, vert_v = 0.0f;

/* Make sure the proper matrices are in the matrix registers */
static void fix_matrices() {
	if (gl_matrix_dirty) {
		mat_identity();
		mat_apply(gl_trans_mats + GL_PROJECTION);
		mat_apply(gl_trans_mats + GL_MODELVIEW);
		gl_matrix_dirty = GL_FALSE;
	}
}

static void end_triangles() {
	int i;
	uint8 *p;
	
	/* Should be three points in the buffer */
	if (dbuf_top != 3) {
		dbglog(DBG_DEBUG, "gl: end_triangles() -- wrong number of vertices\n");
		return;
	}

	/* Throw it all to the TA; expecting clockwise order */
	if (TXR_ENABLED) {
		dbuf_txr[2].flags = TA_VERTEX_EOL;
		p = (uint8*)dbuf_txr;
		fix_matrices();
		mat_transform((vector_t *)(p + 4), (vector_t *)(p + 4), 3, sizeof(vertex_ot_t));
		for (i=0; i<3; i++)
			ta_commit_vertex(dbuf_txr+i, sizeof(vertex_ot_t));
	} else {
		dbuf_col[2].flags = TA_VERTEX_EOL;
		p = (uint8*)dbuf_col;
		fix_matrices();
		mat_transform((vector_t *)(p + 4), (vector_t *)(p + 4), 3, sizeof(vertex_oc_t));
		for (i=0; i<3; i++)
			ta_commit_vertex(dbuf_col+i, sizeof(vertex_oc_t));
	}
}

static void end_quads() {
	int i, j, order[4] = {3, 0, 2, 1};
	uint8 *p;
	
	/* Should be four points in the buffer */
	if (dbuf_top != 4) {
		dbglog(DBG_DEBUG, "gl: end_quads() -- wrong number of vertices\n");
		return;
	}

	/* Throw it all to the TA; expecting clockwise order */
	if (TXR_ENABLED) {
		dbuf_txr[order[3]].flags = TA_VERTEX_EOL;
		p = (uint8*)dbuf_txr;
		fix_matrices();
		mat_transform((vector_t *)(p + 4), (vector_t *)(p + 4), 4, sizeof(vertex_ot_t));
		for (i=0; i<4; i++) {
			j = order[i];
			ta_commit_vertex(dbuf_txr+j, sizeof(vertex_ot_t));
		}
	} else {
		dbuf_col[order[3]].flags = TA_VERTEX_EOL;
		p = (uint8*)dbuf_col;
		fix_matrices();
		mat_transform((vector_t *)(p + 4), (vector_t *)(p + 4), 4, sizeof(vertex_oc_t));
		for (i=0; i<4; i++) {
			j = order[i];
			ta_commit_vertex(dbuf_col+j, sizeof(vertex_oc_t));
		}
	}
}

static void check_end() {
	/* Submit the poly buffer (if neccessary) */
	if (!gl_pbuf_submitted) {
		ta_commit_poly_hdr(gl_pbuf);
		gl_pbuf_submitted = GL_TRUE;
	}
	
	/* What type of primitive was it? */
	switch(gl_prim_type) {
	case GL_TRIANGLES:
		if (dbuf_top == 3) {
			end_triangles();
			dbuf_top = 0;
		}
		break;
	case GL_QUADS:
		if (dbuf_top == 4) {
			end_quads();
			dbuf_top = 0;
		}
		break;
	default:
		dbglog(DBG_DEBUG, "gl: check_end() -- unknown primitive type %d\n", gl_prim_type);
	}
}

void glBegin(GLenum mode) {
	gl_prim_type = mode;
	dbuf_top = 0;
}

void glEnd(void) {
	gl_prim_type = 0;
}

static float autouv[4][2] = {
	{0.0f, 0.0f},
	{1.0f, 0.0f},
	{1.0f, 1.0f},
	{0.0f, 1.0f}
};
void glVertex3f(GLfloat x, GLfloat y, GLfloat z) {
	if (TXR_ENABLED) {
		dbuf_txr[dbuf_top].flags = TA_VERTEX_NORMAL;
		dbuf_txr[dbuf_top].x = x;
		dbuf_txr[dbuf_top].y = y;
		dbuf_txr[dbuf_top].z = z;
		dbuf_txr[dbuf_top].a = vert_a;
		dbuf_txr[dbuf_top].r = vert_r;
		dbuf_txr[dbuf_top].g = vert_g;
		dbuf_txr[dbuf_top].b = vert_b;
		dbuf_txr[dbuf_top].oa =
			dbuf_txr[dbuf_top].or =
			dbuf_txr[dbuf_top].og =
			dbuf_txr[dbuf_top].ob = 0.0f;
		if (AUTOUV_ENABLED) {
			vert_u = autouv[dbuf_top][0];
			vert_v = autouv[dbuf_top][1];
		}
		dbuf_txr[dbuf_top].u = vert_u;
		dbuf_txr[dbuf_top].v = vert_v;
	} else {
		dbuf_col[dbuf_top].flags = TA_VERTEX_NORMAL;
		dbuf_col[dbuf_top].x = x;
		dbuf_col[dbuf_top].y = y;
		dbuf_col[dbuf_top].z = z;
		dbuf_col[dbuf_top].a = vert_a;
		dbuf_col[dbuf_top].r = vert_r;
		dbuf_col[dbuf_top].g = vert_g;
		dbuf_col[dbuf_top].b = vert_b;
	}
	dbuf_top++;

	check_end();
}

void glNormal3f(GLfloat nx, GLfloat ny, GLfloat nz) {
}

void glColor3f(GLfloat red, GLfloat green, GLfloat blue) {
	vert_r = red;
	vert_g = green;
	vert_b = blue;
	vert_a = 1.0f;
}

void glColor4f(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha) {
	vert_r = red;
	vert_g = green;
	vert_b = blue;
	vert_a = alpha;
}

void glTexCoord2f(GLfloat s, GLfloat t) {
	vert_u = s;
	vert_v = t;
}

